
/*
  /-------------------------------------------------------------\
  |                     File : heap.h                           |
  +-------------------------------------------------------------+
  |           Written by : Murat IKINCI                         |
  +-------------------------------------------------------------+
  |           Date       : 24.11.97                             |
  +-------------------------------------------------------------+
  |  Description : Heap                                         |
  +-------------------------------------------------------------+
  |  Modification by :                                          |
  |  Date            :                                          |
  |  Description     :                                          |
  \-------------------------------------------------------------/
*/



/* Heap stores integer ids (must be between minid and maxid) */

typedef EW_type	HeapValType;

typedef struct heap{
  int		    *data;
  int		    *ptr;	    /* index of id in data */
  HeapValType	*vals;      /* associated values of ids (id is index of this array */
  int		    size;
  int		    maxid;
  int           numlocked;
  int           partwgt;
} heap;

typedef heap *pheap;

void HeapAlloc(pheap h, int maxid);
void HeapInit(pheap h, int maxid);
void HeapDeleteAll(pheap h);
void HeapAdd(pheap h, int id);	
void Delete(pheap h, int id);	
void MaxHeapVerify(pheap h);
void MaxHeapInsert(pheap h, int id);
void MaxHeapify(pheap h,int i);
void MaxBuildHeap(pheap h);
int  MaxHeapExtractFirst(pheap h);
void MaxHeapDelete(pheap h, int id);
void MaxHeapIncreaseKey(pheap h, int id);
void Lock(pheap h,int id);

#define MaxHeapDecreaseKey(h,id) (MaxHeapify((h),(h)->ptr[id]))
#define IsHeapEmpty(h)	  ((h)->size==0)
#define HeapGetSize(h)          ((h)->size)
#define HeapGetFirstVal(h)      ((h)->vals[(h)->data[1]])
#define IsInHeap(h, id)         ((h)->ptr[id]!=0)
#define Locked(h,id) ((h)->ptr[id]==-1)
#define Insert(h,id) (HeapAdd(h,id))
#define HeapTop(h) ((h)->data[1])
#define HeapTopVal(h) ((h)->vals[(h)->data[1]])
